#!/usr/bin/env python
# record a TS stream from a DVB device
#

import sys, time, thread
import gobject
import pygst
pygst.require("0.10")
import gst
from optparse import OptionParser

class CLI_Main:

	def __init__(self):
		self.pipeline = gst.Pipeline("pipeline")
		dvb = gst.element_factory_make("dvbbasebin", "dvbbasebin")
		fdsink = gst.element_factory_make("fdsink", "fdsink")
		fdsink.set_property("sync", False)
		self.pipeline.add(dvb, fdsink)
		dvb.get_request_pad("src0")
		gst.element_link_many(dvb, fdsink)
		self.verbose = False
		self.start_time = None
		bus = self.pipeline.get_bus()
		bus.add_signal_watch()
		bus.connect("message", self.on_message)

	def on_message(self, bus, message):
		t = message.type
		if t == gst.MESSAGE_EOS:
			print >> sys.stderr, "Finished by unexpected EOS."
			self.quit()
		elif t == gst.MESSAGE_ERROR:
			err, debug = message.parse_error()
			print >> sys.stderr, "Error: %s" % err
			self.quit()

	def dprint(self, txt):
		if self.verbose:
			print >> sys.stderr, txt

	def quit(self):
		self.pipeline.set_state(gst.STATE_NULL)
		self.playmode = False

	def on_stop_timer(self):
		self.dprint("Stopped the recording.")
		self.quit()
		return False

	def rec(self):
		self.pipeline.set_state(gst.STATE_PLAYING)
		gobject.timeout_add_seconds(self.len, self.on_stop_timer)
		self.dprint("Started to record (for %d [sec.]) ..." % self.len)
		return False

	def main(self):
		self.playmode = True
		if self.start_time is None:
			self.rec()
		else:
			now = time.time()
			if self.start_time - now > 3600 * 6:
				print >> sys.stderr, "start time is too ditant future. quiting..."
				self.playmode = False
			elif now - self.start_time > 3600 * 4:
				print >> sys.stderr, "start time is too distant past. quiting..."
				self.playmode = False
			elif now - self.start_time > -1:
				self.dprint("already past start time.")
				self.rec()
			else:
				gobject.timeout_add_seconds(self.start_time - now, self.rec)
				self.dprint("Waiting about %d sec. to start recording..."
					% int(self.start_time - now))

		while self.playmode:
			time.sleep(1)
		self.loop.quit()

#
# start of the main()
#
gobject.threads_init()
usage = "usage: %prog [options] channel_name length[sec.]"

parser = OptionParser(usage=usage)
parser.add_option("-v", "--verbose",
					action="store_true",
					help="output informational messages. [default: False]")
parser.add_option("-o", "--output",
					dest="filename", metavar="FILE",
					help="write output to FILE. [default: stdout]")

parser.add_option("-a", "--adapter",
					metavar="NUM", type="int",
					help="use DVB adapter #NUM. [default: 0]")
#parser.add_option("-d", "--demuxer",
#					metavar="NUM", type="int",
#					help="use DVB demuxer #NUM. [default: 0]")
parser.add_option("-f", "--frontend",
					metavar="NUM", type="int",
					help="use DVB frontend #NUM. [default: 0]")

#parser.add_option("--conf",
#					metavar="FILE",
#					help="path to the channel config file. "
#							" [default:~/.gstreamer-%d.%d/dvb-channels.conf]"
#							 % gst.version()[0:2])

parser.add_option("-s", "--start",
					metavar="TIME",
					help="start recording at TIME in localtime. [default: now]  "
							"Format of TIME: %Y-%m-%dT%H:%M:%S")


(options, args) = parser.parse_args()
try:
	mainclass = CLI_Main()

	if len(args) < 2:
		parser.error("ch.name and length args are required.")

	if options.filename:
		try:
			f = open(options.filename, 'w')
			mainclass.pipeline.get_by_name("fdsink").set_property("fd", f.fileno())
		except:
			parser.error("option -o has unwritable filename: %s" % options.filename)

	if options.adapter:
		mainclass.pipeline.get_by_name("dvbbasebin").set_property("adapter", options.adapter)
	if options.frontend:
		mainclass.pipeline.get_by_name("dvbbasebin").set_property("frontend", options.frontend)

	if options.start:
		mainclass.start_time = time.mktime(
			time.strptime(options.start, "%Y-%m-%dT%H:%M:%S"))

	len = int(args[1], 0)
	if len <= 0:
		parser.error("invalid length arg.:%d." % len)
	mainclass.len = len

	mainclass.pipeline.get_by_name("dvbbasebin").set_uri(
			"dvb://%s" % args[0])
	mainclass.verbose = options.verbose

	loop = gobject.MainLoop()
	mainclass.loop = loop
	thread.start_new_thread(mainclass.main, ())
	loop.run()
except:
	mainclass.quit()
	print >> sys.stderr, "aborted by:%s, %s" % sys.exc_info()[0:2]
